/*
	-> Este é um arquivo base para desenvolvimento para a criação da dependência de camadas
		da aplicação, utilizada no pacote principal e de testes;
	-> Codifique em inglês utilizando os termos definidos no glossário;
	-> Qualquer dúvida contate um AGES 3.
*/
package factory

import (
	"institutionMS/layers/api"
	"institutionMS/layers/infrastructure"
	"institutionMS/layers/repository"
	"institutionMS/layers/service"
	"os"
)

// CreateInstitutionApp creates application router through layer dependency chain
func CreateInstitutionApp() (router api.Api) {
	db := createInfrastructure()
	repo := createRepository(db)
	svc := createService(repo)
	router = createApi(svc)
	return
}

// createInfrastructure creates an database for repository layer
func createInfrastructure() (db infrastructure.DocumentDatabase) {
	db = func() *infrastructure.MongoDB {
		return &infrastructure.MongoDB{
			Username: os.Getenv("MONGO_USER"),
			Password: os.Getenv("MONGO_PASSWORD"),
			Host:     os.Getenv("MONGO_HOST"),
			Port:     os.Getenv("MONGO_PORT"),
		}
	}()
	return
}

// createRepository creates a repository for service layer
func createRepository(db infrastructure.DocumentDatabase) (repo repository.Repository) {
	repo = repository.InstitutionRepository{
		Database:                 db,
		TargetDocumentDatabase:   os.Getenv("MONGO_DATABASE"),
		TargetDocumentCollection: os.Getenv("MONGO_COLLECTION_NAME"),
	}
	return
}

// createService creates a service for api layer
func createService(repo repository.Repository) (svc service.Service) {
	svc = service.InstitutionService{
		Repository: repo,
	}
	return
}

// createApi creates an api router
func createApi(svc service.Service) (router api.Api) {
	router = api.InstitutionApi{
		Service: svc,
	}
	return
}
