/*
	-> Este é um arquivo base para desenvolvimento na camada de Serviço;
	-> Codifique em inglês utilizando os termos definidos no glossário;
	-> Qualquer dúvida contate um AGES 3.
*/
package service

import (
	"institutionMS/layers/entity"
	"institutionMS/layers/repository"
	"net/http"

	"github.com/gin-gonic/gin"
)


// InstitutionService is a struct that defines a institution Service behaviour at service layer
type InstitutionService struct {
	Repository repository.Repository
}

// setStatusOk sets OK status to a gin context
func (service InstitutionService) setStatusOk(msg interface{}, ctx *gin.Context) {
	ctx.JSON(http.StatusOK, msg)
}

// setStatusBadRequest sets BAD REQUEST status to a gin context
func (service InstitutionService) setStatusBadRequest(msg interface{}, ctx *gin.Context) {
	ctx.JSON(http.StatusBadRequest, msg)
}

// HandleGet is a business logic function that handles Institution GET HTTP method
func (service InstitutionService) HandleGet(ctx *gin.Context) {
	id, _ := ctx.Params.Get("id")
	institution := service.Repository.Load(id)

	if institution == nil {
		service.setStatusBadRequest(gin.H{"error": "Institution not found."}, ctx)
	} else {
		service.setStatusOk(institution, ctx)
	}
}

// HandlePost is a business logic function that handles Institution POST HTTP method
func (service InstitutionService) HandlePost(ctx *gin.Context) {
	insittution := new(entity.Institution)
	err := ctx.Bind(insittution)
	if err != nil {
		service.setStatusBadRequest(gin.H{"error": error.Error(err)}, ctx)
	} else {
		id := service.Repository.Save(*insittution)
		service.setStatusOk(gin.H{"id": id}, ctx)
	}
}

// HandleUpdate is a business logic function that handles Institution PUT HTTP method
func (service InstitutionService) HandleUpdate(ctx *gin.Context) {
	id, _ := ctx.Params.Get("id")
	institution := new(entity.Institution)
	err := ctx.Bind(institution)
	if err != nil {
		service.setStatusBadRequest(gin.H{"error": error.Error(err)}, ctx)
	} else {
		updated, err := service.Repository.Update(id, institution)
		if err != nil {
			service.setStatusBadRequest(gin.H{"error": error.Error(err)}, ctx)
		} else {
			service.setStatusOk(updated, ctx)
		}
	}
}
