/*
	-> Este é um arquivo base para desenvolvimento de testes da aplicação;
	-> Codifique em inglês utilizando os termos definidos no glossário;
	-> Qualquer dúvida contate um AGES 3.
*/
package test

import (
	"bytes"
	"institutionMS/factory"
	"log"
	"net/http"
	"net/http/httptest"
	"testing"

	"github.com/gin-gonic/gin"
	"github.com/joho/godotenv"
	"github.com/stretchr/testify/assert"
)

var idCreatedByPost string
var idCreatedByPostWithQuotation string

// setupRouter is a private function that loads Gin router
func setupRouter() (ginRouter *gin.Engine) {
	err := godotenv.Load("../.env")

	if err != nil {
		log.Fatal("Error loading .env file")
	}

	ginRouter = gin.Default()
	appRouter := factory.CreateInstitutionApp()
	appRouter.Create(ginRouter)
	return
}

// TestSuccessInstitutionPost tests Institution's POST method with success response
func TestSuccessInstitutionPost(t *testing.T) {
	router := setupRouter()

	json := []byte(`{"IsValid": true,
		"Password": "123",
		"Name": "bla",
		"Email": "bla@email.com",
		"Description": "Teste",
		"Location" : { "Complement": "B", "Number" : 3, "Street": "Brasiliano", "City": "Porto Alegre", "State": "Rio Grande do Sul", "Country": "Brazil"} }`)

	w := httptest.NewRecorder()
	req, _ := http.NewRequest("POST", "/institution", bytes.NewBuffer(json))
	req.Header.Add("Content-Type", "application/json")
	router.ServeHTTP(w, req)

	println(w.Code)
	println(w.Body.String())
	idCreatedByPost = w.Body.String()
	idCreatedByPostWithQuotation = w.Body.String()
	idCreatedByPostWithQuotation= string(idCreatedByPostWithQuotation[0:len(idCreatedByPostWithQuotation)-1])
	idCreatedByPost = idCreatedByPost[7:len(idCreatedByPost)-2]

	assert.Equal(t, http.StatusOK, w.Code)
	assert.Contains(t, w.Body.String(), "id")
}

// TestSuccessInstitutionGet tests Institution's GET method with success response
func TestSuccessInstitutionGet(t *testing.T) {
	router := setupRouter()
	w := httptest.NewRecorder()
	req, _ := http.NewRequest("GET", "/institution/"+idCreatedByPost, nil)
	router.ServeHTTP(w, req)

	assert.Equal(t, http.StatusOK, w.Code)
	assert.Equal(t, idCreatedByPostWithQuotation+`,"isvalid":true,"password":"123","name":"bla","email":"bla@email.com","description":"Teste","location":{"complement":"B","number":3,"street":"Brasiliano","city":"Porto Alegre","state":"Rio Grande do Sul","country":"Brazil"}}`, w.Body.String())
}

// TestSuccessInstitutionPut tests Institution's PUT method with success response
func TestSuccessInstitutionPut(t *testing.T) {
	router := setupRouter()

	json := []byte(`{"IsValid": true,
		"Password": "123",
		"Name": "blaa",
		"Email": "bla@email.com",
		"Description": "Teste",
		"Location" : { "Complement": "B", "Number" : 3, "Street": "Brasiliano", "City": "Porto Alegre", "State": "Rio Grande do Sul", "Country": "Brazil"} }`)

	w := httptest.NewRecorder()
	req, _ := http.NewRequest("PUT", "/institution/"+idCreatedByPost, bytes.NewBuffer(json))
	req.Header.Add("Content-Type", "application/json")
	router.ServeHTTP(w, req)

	assert.Equal(t, http.StatusOK, w.Code)
	assert.Equal(t, idCreatedByPostWithQuotation+`,"isvalid":true,"password":"123","name":"blaa","email":"bla@email.com","description":"Teste","location":{"complement":"B","number":3,"street":"Brasiliano","city":"Porto Alegre","state":"Rio Grande do Sul","country":"Brazil"}}`, w.Body.String())
}
